import { NotFoundException } from '@nestjs/common';
import { QueryHandler } from '@nestjs/cqrs';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { Messager } from '../../adapters/secondaries/messager';
import { FindTerritoryByUuidQuery } from '../../queries/find-territory-by-uuid.query';
import { Territory } from '../entities/territory';

@QueryHandler(FindTerritoryByUuidQuery)
export class FindTerritoryByUuidUseCase {
  constructor(
    private readonly _repository: TerritoriesRepository,
    private readonly _messager: Messager,
  ) {}

  async execute(
    findTerritoryByUuidQuery: FindTerritoryByUuidQuery,
  ): Promise<Territory> {
    try {
      const territory = await this._repository.findOneByUuid(
        findTerritoryByUuidQuery.uuid,
      );
      if (!territory) throw new NotFoundException();
      return territory;
    } catch (error) {
      this._messager.publish(
        'logging.territory.read.warning',
        JSON.stringify({
          query: findTerritoryByUuidQuery,
          error,
        }),
      );
      throw error;
    }
  }
}
