import { RabbitMQModule } from '@golevelup/nestjs-rabbitmq';
import { RedisClientOptions } from '@liaoliaots/nestjs-redis';
import { CacheModule, Module } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { CqrsModule } from '@nestjs/cqrs';
import { redisStore } from 'cache-manager-ioredis-yet';
import { DatabaseModule } from '../database/database.module';
import { TerritoriesController } from './adapters/primaries/territories.controller';
import { TerritoriesRepository } from './adapters/secondaries/territories.repository';
import { Messager } from './adapters/secondaries/messager';
import { CreateTerritoryUseCase } from './domain/usecases/create-territory.usecase';
import { DeleteTerritoryUseCase } from './domain/usecases/delete-territory.usecase';
import { FindAllTerritoriesForPointUseCase } from './domain/usecases/find-all-territories-for-point.usecase';
import { FindAllTerritoriesForPointsUseCase } from './domain/usecases/find-all-territories-for-points.usecase';
import { FindAllTerritoriesUseCase } from './domain/usecases/find-all-territories.usecase';
import { FindTerritoryByUuidUseCase } from './domain/usecases/find-territory-by-uuid.usecase';
import { UpdateTerritoryUseCase } from './domain/usecases/update-territory.usecase';
import { TerritoryProfile } from './mappers/territory.profile';

@Module({
  imports: [
    DatabaseModule,
    CqrsModule,
    RabbitMQModule.forRootAsync(RabbitMQModule, {
      imports: [ConfigModule],
      useFactory: async (configService: ConfigService) => ({
        exchanges: [
          {
            name: configService.get<string>('RMQ_EXCHANGE'),
            type: 'topic',
          },
        ],
        uri: configService.get<string>('RMQ_URI'),
        connectionInitOptions: { wait: false },
      }),
      inject: [ConfigService],
    }),
    CacheModule.registerAsync<RedisClientOptions>({
      imports: [ConfigModule],
      useFactory: async (configService: ConfigService) => ({
        store: await redisStore({
          host: configService.get<string>('REDIS_HOST'),
          port: configService.get<number>('REDIS_PORT'),
          ttl: configService.get<number>('CACHE_TTL'),
        }),
      }),
      inject: [ConfigService],
    }),
  ],
  controllers: [TerritoriesController],
  providers: [
    TerritoryProfile,
    TerritoriesRepository,
    Messager,
    FindAllTerritoriesForPointUseCase,
    FindAllTerritoriesForPointsUseCase,
    FindAllTerritoriesUseCase,
    FindTerritoryByUuidUseCase,
    CreateTerritoryUseCase,
    UpdateTerritoryUseCase,
    DeleteTerritoryUseCase,
  ],
  exports: [],
})
export class TerritoriesModule {}
