import { Test, TestingModule } from '@nestjs/testing';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { FindAllTerritoriesForPointRequest } from '../../domain/dtos/find-all-territories-for-point.request';
import { FindAllTerritoriesForPointUseCase } from '../../domain/usecases/find-all-territories-for-point.usecase';
import { FindAllTerritoriesForPointQuery } from '../../queries/find-all-territories-for-point.query';

const findAllTerritoriesForPointRequest: FindAllTerritoriesForPointRequest =
  new FindAllTerritoriesForPointRequest();
findAllTerritoriesForPointRequest.lon = 6.181455;
findAllTerritoriesForPointRequest.lat = 48.685689;

const findforPointQuery: FindAllTerritoriesForPointQuery =
  new FindAllTerritoriesForPointQuery(findAllTerritoriesForPointRequest);

const mockTerritories = [
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
    name: 'Nancy',
  },
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a92',
    name: 'Meurthe-et-Moselle',
  },
];

const mockTerritoriesRepository = {
  findForPoint: jest
    .fn()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    .mockImplementation((query?: FindAllTerritoriesForPointQuery) => {
      return Promise.resolve(mockTerritories);
    }),
};

describe('FindAllTerritoriesforPointUseCase', () => {
  let findAllTerritoriesForPointUseCase: FindAllTerritoriesForPointUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: TerritoriesRepository,
          useValue: mockTerritoriesRepository,
        },
        FindAllTerritoriesForPointUseCase,
      ],
    }).compile();

    findAllTerritoriesForPointUseCase =
      module.get<FindAllTerritoriesForPointUseCase>(
        FindAllTerritoriesForPointUseCase,
      );
  });

  it('should be defined', () => {
    expect(findAllTerritoriesForPointUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should return an array filled with territories', async () => {
      const territories = await findAllTerritoriesForPointUseCase.execute(
        findforPointQuery,
      );

      expect(territories).toBe(mockTerritories);
    });
  });
});
